#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <sys/disklabel.h>

/* ZIP_disklabel -- reads and rewrites Digital Unix Disklabels */
/*  specifically for Iomega ZIP drives */

int main(int argc, char * argv[], char * env[])
{
int temp;
int x;
int error;
unsigned char buffer[512];
int f_handle;
struct disklabel * dl;

/* This program requires two arguements as follows */

if (argc!=3)
	{
	printf("Usage: ZIP_disklabel -r|-w device\n");
	printf("\n -r: Read the disklabel from an existing ZIP disk\n");
	printf("     and write it to the file ZIP_bin_disklabel.\n");
	printf("\n -w: Write the disk label.\n");
	exit(0);
	}
	
/* Routine to read a disklabel */

if (strcmp("-r",argv[1])==0)
	{
	
	/* read the label */
	
	printf("Reading Disklabel...\n");
	f_handle=open(argv[2],O_RDONLY,0666);
	if (f_handle==-1)
		{
		perror("Cannot open device for reading");
		exit(0);
		}
	read(f_handle,buffer,512);
	close(f_handle);

	/* Write the blabel to a file */
	
	printf("Saving image...\n");
	f_handle=open("ZIP_bin_disklabel",O_CREAT|O_TRUNC|O_WRONLY,0666);
	if (f_handle==-1)
		{
		perror("Cannot open ZIP_bin_disklabel for writing");
		exit(0);
		}
	write(f_handle,buffer,512);
	close(f_handle);
	printf("Done.\n");
	exit(0);
	}

/* Routine to write a disklabel */

if (strcmp("-w",argv[1])==0)
	{
	
	/* Check to see what is on the disk */
	
	printf("Preparing to write Disklabel...\n");
	f_handle=open(argv[2],O_RDONLY,0666);
	if (f_handle==-1)
		{
		perror("Cannot open ZIP_bin_disklabel for reading");
		exit(0);
		}
	read(f_handle,buffer,512);
	close(f_handle);
	printf("Reading Block 0...\n\n");
	for (temp=0;temp<16;temp++)
		{
		printf(" 0x%3.3X: ",temp*32);
		for (x=0;x<32;x++)
			{
			printf("%2.2X",(unsigned int)buffer[temp*32+x]);
			/*printf("%c",(unsigned int)buffer[temp*32+x]);*/
			}
		printf("\n");
		}
	dl = (struct disklabel *)&buffer[0x40];
		
	/* Verify user wants to continue */
	
	printf("\nContinue (y/n)? ");
	scanf("%s",buffer);
	if ((buffer[0]!='y') && (buffer[0]!='Y'))
		{
		printf("Aborting\n");
		exit(0);
		}
		
	/* Read the image file */
			
	printf("Reading image....\n");
	f_handle=open("ZIP_bin_disklabel",O_RDONLY,0666);
	if (f_handle==-1)
		{
		perror("Cannot open ZIP_bin_disklabel for reading");
		exit(0);
		}
	read(f_handle,buffer,512);
	close(f_handle);
	
	/* Write the label */
	
	printf("Writing disklabel...\n");
	f_handle=open(argv[2],O_WRONLY,0666);
	if (f_handle==-1)
		{
		perror("Cannot open device for writing disklabel");
		exit(0);
		}
	write(f_handle,buffer,512);
	close(f_handle);
	printf("Done.\n");
	exit(0);
	}
}
